'use strict';

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _this = undefined;

var addic7ed = require("addic7ed-api");

var transform = function transform(query, items) {
  var results = [];

  items.map(function (item) {
    var result = {
      name: query + "." + item.distribution + "." + item.team,
      download: item,
      extention: "",
      source: "addic7ed",
      size: "",
      score: 0
    };

    results.push(result);
  });

  return results;
};

var textSearch = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(query, language, limit) {
    var splitQuery, subtitles, serie, season, episode, items;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            splitQuery = query.match(/s([0-9]{1,2})\s*e([0-9]{1,2})/i);

            if (splitQuery) {
              _context.next = 4;
              break;
            }

            console.log("Addic7ed: Can't parse " + query + "...");
            return _context.abrupt("return", []);

          case 4:
            subtitles = [];
            serie = query.replace(splitQuery[0], "");
            season = parseInt(splitQuery[1], 10);
            episode = parseInt(splitQuery[2], 10);
            _context.next = 10;
            return addic7ed.search(serie, season, episode, language);

          case 10:
            items = _context.sent;

            if (items) {
              _context.next = 14;
              break;
            }

            console.log("Addic7ed: Nothing found...");
            return _context.abrupt("return", []);

          case 14:
            return _context.abrupt("return", transform(query, items));

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, _this);
  }));

  return function textSearch(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var fileSearch = function () {
  var _ref2 = asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(files, language, limit) {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, _this);
  }));

  return function fileSearch(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

var addic7ed$1 = {
  fileSearch: fileSearch,
  download: download,
  textSearch: textSearch
};

var _this$1 = undefined;

var OS = require("opensubtitles-api");

var _require$1 = require("lodash");
var head = _require$1.head;

var OpenSubtitles = new OS("caption");

var transform$1 = function transform(items) {
  var results = [];

  items.map(function (item) {
    var result = {
      name: item.filename,
      download: item.url,
      extention: "",
      source: "opensubtitles",
      size: "",
      score: item.score
    };

    results.push(result);
  });

  return results;
};

var textSearch$1 = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(query, language, limit) {
    var options, items, firstItem, results;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            options = {
              sublanguageid: language,
              limit: limit,
              query: query
            };
            _context.next = 3;
            return OpenSubtitles.search(options);

          case 3:
            items = _context.sent;

            if (items) {
              _context.next = 7;
              break;
            }

            console.log("Opensubtitles: Nothing found...");
            return _context.abrupt("return", []);

          case 7:
            firstItem = head(Object.keys(items)); // firstItem is selected language: obj[language]

            results = items[firstItem];

            if (results) {
              _context.next = 11;
              break;
            }

            return _context.abrupt("return", []);

          case 11:
            return _context.abrupt("return", transform$1(results));

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, _this$1);
  }));

  return function textSearch(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var fileSearch$1 = function () {
  var _ref2 = asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(files, language, limit) {
    var subtitleReferences, downloadedReferences, subtitleResults;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            subtitleReferences = files.map(function () {
              var _ref3 = asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(file) {
                var info, options, result, firstItem, subtitle;
                return regeneratorRuntime.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        _context2.next = 2;
                        return OpenSubtitles.identify({
                          path: file.path,
                          extend: true
                        });

                      case 2:
                        info = _context2.sent;
                        options = {
                          sublanguageid: language,
                          limit: limit,
                          hash: info.moviehash,
                          filesize: info.moviebytesize,
                          path: file.path,
                          filename: file.filename
                        };


                        if (info && info.metadata && info.metadata.imdbid) {
                          options["imdbid"] = info.metadata.imdbid;
                        }

                        _context2.next = 7;
                        return OpenSubtitles.search(options);

                      case 7:
                        result = _context2.sent;
                        firstItem = head(Object.keys(result));
                        subtitle = result[firstItem];
                        return _context2.abrupt("return", {
                          file: file,
                          subtitle: subtitle
                        });

                      case 11:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2, _this$1);
              }));

              return function (_x7) {
                return _ref3.apply(this, arguments);
              };
            }());
            _context3.next = 3;
            return Promise.all(subtitleReferences);

          case 3:
            downloadedReferences = _context3.sent;
            subtitleResults = downloadedReferences.filter(function (_ref4) {
              var subtitle = _ref4.subtitle;
              return subtitle !== undefined;
            });
            return _context3.abrupt("return", subtitleResults);

          case 6:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, _this$1);
  }));

  return function fileSearch(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

var opensubtitles = {
  textSearch: textSearch$1,
  fileSearch: fileSearch$1
};

var Promise$1 = require("bluebird");

var _require = require("lodash");
var flatMap = _require.flatMap;

var CaptionCore = function () {
  function CaptionCore() {
    classCallCheck(this, CaptionCore);

    this.sources = [addic7ed$1, opensubtitles];
  }

  createClass(CaptionCore, [{
    key: "searchByQuery",
    value: function searchByQuery(query) {
      var language = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "eng";
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;

      var checkSources = this.sources.map(function (source) {
        return source.textSearch(query, language, limit);
      });

      return {
        on: function on(event, callback) {
          switch (event) {
            case "fastest":
              // Wait for first source to finish downloading, return first set of results to renderer.
              Promise$1.any(checkSources).then(function (results) {
                callback(results);
              });

              return this;

            case "completed":
            default:
              // Check whether all sources have been checked. Return to renderer and hide loading spinner.
              Promise$1.all(checkSources).then(function (results) {
                var emptyResultSet = results.filter(function (result) {
                  return result.length > 0;
                });

                if (emptyResultSet.length === 0) {
                  callback([]);
                }

                callback(flatMap(results));
              });

              return this;
          }
        }
      };
    }
  }, {
    key: "searchByFiles",
    value: function searchByFiles() {
      var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var language = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "eng";
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;

      var opensubtitlesRef = opensubtitles.fileSearch(files, language, limit);

      return {
        on: function on(event, callback) {
          switch (event) {
            case "completed":
            default:
              // First promise which is resolved should return its results
              Promise$1.race([opensubtitlesRef]).then(function (results) {
                return callback(results);
              });

              return this;
          }
        }
      };
    }
  }]);
  return CaptionCore;
}();

module.exports = new CaptionCore();
